import { useEffect, useReducer, useState } from 'react';
import selectn from 'selectn';
import { addCustomEventListener, removeCustomEventListener } from '../../../utils/customEventDispatcher';
import { DUMMY_ARRAY } from '../../../utils/constants';
import { DOT_SERVICE_EVENT, DOT_SERVICE_EVENT_TYPE_HIDE_TOAST_NOTIFICATION, DOT_SERVICE_EVENT_TYPE_SHOW_PROMISE_TOAST_NOTIFICATION, DOT_SERVICE_EVENT_TYPE_UPDATE_PROMISE_TOAST_NOTIFICATION, DOT_SERVICE_EVENT_TYPE_HIDE_PROMISE_TOAST_NOTIFICATION, DOT_SERVICE_EVENT_TYPE_SHOW_TOAST_NOTIFICATION } from '../../../service/helpers/constant';
import toastNotificationReducer from '../reducers/toastNotificationReducer';
let toastIdCounter = 0;

const useDOTToastNotificationService = () => {
  const [notifications, dispatch] = useReducer(toastNotificationReducer);

  const handleService = event => {
    if (selectn('detail.payload.promise', event)) {
      handlePromiseToast({
        dispatch,
        ...selectn('detail.payload', event)
      });
    } else {
      const type = selectn('detail.type', event);
      const payload = selectn('detail.payload', event);
      const id = `toast${toastIdCounter++}`;
      const toastMessage = {
        type,
        payload: {
          id,
          ...payload
        }
      };
      dispatch(toastMessage);
    }
  };

  const handlePromiseToast = _ref => {
    let {
      dispatch,
      promise,
      success,
      error,
      loading,
      onStatusChange
    } = _ref;
    const id = `promiseToast${toastIdCounter++}`;
    const loadingToast = {
      id,
      type: 'info',
      ...loading
    };

    if (typeof onStatusChange == 'function') {
      loadingToast.onStatusChange = onStatusChange;
    }

    dispatch({
      type: DOT_SERVICE_EVENT_TYPE_SHOW_PROMISE_TOAST_NOTIFICATION,
      payload: loadingToast
    });
    promise.then(() => {
      const successToast = {
        id,
        type: 'success',
        ...success
      };

      if (typeof onStatusChange == 'function') {
        successToast.onStatusChange = onStatusChange;
      }

      dispatch({
        type: DOT_SERVICE_EVENT_TYPE_UPDATE_PROMISE_TOAST_NOTIFICATION,
        payload: successToast
      }); // setTimeout(() => {
      //     dispatch({
      //         type: DOT_SERVICE_EVENT_TYPE_HIDE_PROMISE_TOAST_NOTIFICATION
      //     })
      // }, 1000);
    }).catch(() => {
      const errorToast = {
        id,
        type: 'error',
        ...error
      };

      if (typeof onStatusChange == 'function') {
        errorToast.onStatusChange = onStatusChange;
      }

      dispatch({
        type: DOT_SERVICE_EVENT_TYPE_UPDATE_PROMISE_TOAST_NOTIFICATION,
        payload: errorToast
      }); // setTimeout(() => {
      //     dispatch({
      //         type: DOT_SERVICE_EVENT_TYPE_HIDE_PROMISE_TOAST_NOTIFICATION
      //     })
      // }, 1000);
    });
  };

  useEffect(() => {
    addCustomEventListener({
      eventName: DOT_SERVICE_EVENT,
      callback: handleService
    });
    return () => {
      removeCustomEventListener({
        eventName: DOT_SERVICE_EVENT,
        callback: handleService
      });
    };
  }, []);
  return {
    notifications
  };
};

export default useDOTToastNotificationService;